--[[
Shards Server version: 0.3.4
Author: Nathaniel Staples
Date Modified: 29/08/2016

This file contains  code for the global scene controler that can be added to a seed object to
load all custom scenes from the mods\modName\mapdate\worldName\Scenes on initial
server up (empty backups folder)

Massive Credit to Yorlik of Arcanima for xml code.
--]]

--------------------------------------------
--Variables
--------------------------------------------

--Your mod name goes here
local myMod = "SceneCreator"

--------------------------------------------
--Functions
--------------------------------------------



--Custom function to split a string into a table of each of its components.
--inputString = string - the string to be split
--seperator the charcter (or charater string to split on) these are not included in the output.
function splitString(inputString, seperator)
    if separator == nil then
		separator = "%s"
	end
		
	local stringTable={} ; i=1
	for str in string.gmatch(inputString, "([^"..seperator.."]+)") do
		stringTable[i] = str
		i = i + 1
	end
	return stringTable
end


-- Initiate scene spawner on fresh server load
RegisterSingleEventHandler(EventType.ModuleAttached, "NS_scene_spawn_controller",
	function ()
		this:ScheduleTimerDelay(TimeSpan.FromSeconds(10),"spawnSceneObjects")
--		DebugMessage("we are here")
	end)

	
-- Wait until beginning scene spawning
	RegisterEventHandler(EventType.Timer,"spawnSceneObjects",
	function ()
		this:SendMessage("spawnSceneObjects")
--		DebugMessage("we are here")
	end)


-- spawns all objects for all scene files in mapdate\worldName\Scenes  directory
RegisterEventHandler(EventType.Message,"spawnSceneObjects",
	function ()
		----------------------------------------
		-- Set directory path to searched for xml scene files
		-- Import scene list from directory mods\modName\mapdate\worldName\Scenes
		----------------------------------------

		sceneDirectory = "mods\\".. tostring(myMod).. "\\mapdata\\"..GetWorldName().."\\Scenes\\" ..  "*.xml"

		local ioStream = io.popen("dir \"" .. tostring(sceneDirectory) .. "\"")
		if ioStream then
--			DebugMessage(ioStream:read("*a"))
			directoryInfo = ioStream:read("*a")
--			DebugMessage(directoryInfo)
		else
			DebugMessage("failed to read")
		end

		scenesToSpawn ={}

		for fileName in string.gmatch(directoryInfo, "%g+.xml") do 
			table.insert(scenesToSpawn, fileName)
		end	
--		DebugMessage(DumpTable(scenesToSpawn))

		----------------------------------------
		-- Cycle through list of scene files. Spawn the objects for each scene!
		----------------------------------------

		for i, sceneFileName in pairs(scenesToSpawn) do
			local scene = sceneFileName
		
			sceneFile = "mods\\".. tostring(myMod).. "\\mapdata\\"..GetWorldName().."\\Scenes\\" .. tostring(scene)
--			DebugMessage ("Scenefile is " .. sceneFile)

			----------------------------------------
			-- Convert the xml file into a lua table
			----------------------------------------
		
			local sceneObjectList = xml.load(sceneFile)
--			DebugMessage(DumpTable(sceneObjectList))

			----------------------------------------
			-- Extract all the object info from the file. Use template, location, and scale to create item
			-- send the rest with create function to ve handled by the item created event message
			----------------------------------------
			
			for i = 1, #sceneObjectList do
--				DebugMessage("Scanning object :", i)

				local objectInfo ={}
				object = sceneObjectList[i]

				for i = 1, #object do
--					DebugMessage("Scanning attribute :", i)
					attribute = object[i]

--					DebugMessage(attribute[0])
--					DebugMessage(attribute[1])

					objectInfo[attribute[0]] = attribute[1]
				end	

				-- covert stored location into usful information
				local loc = splitString(objectInfo.ObjectLocation, ",")
				
--				DebugMessage(loc[1], loc[2], loc[3])
				local tempLocation = Loc(tonumber(loc[1]), tonumber(loc[2]), tonumber(loc[3]))
				objectInfo.ObjectLocation = tempLocation
				
				-- covert stored scale Vector into usful information
				if (objectInfo.ObjectScaleVector ~= nil) then
					local scale = splitString(objectInfo.ObjectScaleVector, " ")
				
					local tempScale = Loc(tonumber(scale[1]), tonumber(scale[2]), tonumber(scale[3]))
					objectInfo.ScaleVector = tempScale

--					DebugMessage(DumpTable(objectInfo))
--					DebugMessage(objectInfo.Template)
--					DebugMessage(tempLocation)

--					CreateObj(objectInfo.Template, objectInfo.ObjectLocation, "sceneObjectSpawned", objectInfo)
					CreateObjExtended(objectInfo.Template, nil, objectInfo.ObjectLocation, Loc(0,0,0), objectInfo.ScaleVector, "sceneObjectSpawned", objectInfo)
				end
			end
		end
	end)


-- Attaches all scripts and objVars to created scene opjects.	
RegisterEventHandler(EventType.CreatedObject,"sceneObjectSpawned",
	function(success, objref, objectInfo)
		----------------------------------------
		-- Set standard values to stop players interacting with objects
		----------------------------------------
		objref:SetSharedObjectProperty("NoInteract", true)
		objref:SetSharedObjectProperty("Weight",-1)		
		objref:SetObjVar("handlesPickup",true)

		----------------------------------------
		-- Set values retrieved from XML file passed in objectInfo
		-- First retrieve custom scripts and attach them
		-- second get objVats and set them
		----------------------------------------

		-- covert stored rotation into usful information
		local rot = splitString(objectInfo.ObjectRotation, ",")
		local tempRotation = Loc(tonumber(rot[1]), tonumber(rot[2]), tonumber(rot[3]))
		objectInfo.ObjectRotation = tempRotation

		objref:SetHue(objectInfo.Hue)
		objref:SetRotation(objectInfo.ObjectRotation)
		objref:SetName(objectInfo.Name)

		-- Attach Scripts
		if objectInfo.CustomScripts ~= nil and objectInfo.CustomScripts ~= "" then
			local scriptsToAttach = splitString(objectInfo.CustomScripts, ",")
			for i, script in pairs(scriptsToAttach) do
				objref:AddModule(script)
			end
		end
		
		-- Attach object variables
		local objVarsToAttach = splitString(objectInfo.ObjVars, ";")
		for i = 1, #objVarsToAttach do
			objVarPair = splitString(objVarsToAttach[i], "=")

--			DebugMessage(objVarPair[1], objVarPair[2])
			if objVarPair[3] == "string" then
				objref:SetObjVar(objVarPair[1], objVarPair[2])
			elseif 	objVarPair[3] == "number" then
				objref:SetObjVar(objVarPair[1], tonumber(objVarPair[2]))
			else
				if objVarPair[2] == "true" then
					objref:SetObjVar(objVarPair[1], true)
				else
					objref:SetObjVar(objVarPair[1], false )
				end
			end
		end
	DebugMessage("Object created!!")
	end)
	