--[[
modded scriptcommands.lua

Shards Server version: 0.3.4
Author: Nathaniel Staples
Date Modified: 29/08/2016


Due to a variable being declared local in the vanilla code the whole file needs to be imported and over ridden.
The require statement imports the local variable curtab that can not be accessed or overridden by
another file.

--]]


------------------------------------
-- Origional File
------------------------------------

-- INFO WINDOW

function GetObjVarsSorted(gameObj)
	local array = {}
	for key, value in pairs(gameObj:GetAllObjVars()) do
		table.insert(array,{name = key, value = value})		
	end
	table.sort(array,function(a,b) return a.name < b.name end)

	return array
end

curInfoObj = this
local curTab = "Behaviors"
local selObjVar = nil
function DoInfo(target)
    curInfoObj = target    
 
    local newWindow = DynamicWindow("InfoWindow","Object Info ("..curInfoObj.Id..")",440,500)
 
    newWindow:AddLabel(20, 10, "[F3F781]Name: [-]"..(curInfoObj:GetName() or ""),600,0,18,"left",false)
    newWindow:AddLabel(20, 30, "[F3F781]Loc: [-]" ..tostring(curInfoObj:GetLoc()),600,0,18,"left",false)
    newWindow:AddLabel(20, 50, "[F3F781]Template: [-]"..tostring(curInfoObj:GetCreationTemplateId()),600,0,18,"left",false)
    newWindow:AddLabel(305, 50, "[F3F781]ClientId: [-]"..tostring(curInfoObj:GetIconId()),600,0,18,"left",false)
    newWindow:AddLabel(20, 70, "[F3F781]Cloaked: [-]"..tostring(curInfoObj:IsCloaked()),600,0,18,"left",false)
    newWindow:AddLabel(115, 70, "[F3F781]Frozen: [-]"..tostring(curInfoObj:IsMobileFrozen()),600,0,18,"left",false)
    newWindow:AddLabel(210, 70, "[F3F781]Hue: [-]"..tostring(curInfoObj:GetHue()),600,0,18,"left",false)
    if curInfoObj:IsMobile() then
        newWindow:AddLabel(305, 70, "[F3F781]Scale: [-]"..string.format("%4.2f",curInfoObj:GetScale()),600,0,18,"left",false)
    end
    newWindow:AddButton(320, 10, "Refresh", "Refresh", 80, 23, "", "", false,"SimpleButton")
 
    local behaviorState = ""
    if(curTab == "Behaviors") then
        behaviorState = "pressed"
    end
    local objvarState = ""
    if(curTab == "ObjVars") then
        objvarState = "pressed"
    end
    newWindow:AddButton(20,100,"BehaviorsTab","Behaviors",190,23,"","",false,"SimpleButton",behaviorState)
    newWindow:AddButton(210,100,"ObjVarsTab","Object Variables",190,23,"","",false,"SimpleButton",objvarState)
 
    --newWindow:AddLabel(20, 100, "[F3F781]Behaviors:[-]",0,0,18,"left",true)
    newWindow:AddImage(20,130,"FullBorder",380,310,"Sliced")
 
    if(curTab == "Behaviors") then 
        local scrollWindow = ScrollWindow(25,135,355,225,25)
        for i,behavior in pairs(curInfoObj:GetAllModules()) do
            scrollElement = ScrollElement()
            if((i-1) % 2 == 1) then
            scrollElement:AddImage(0,0,"Blank",320,25,"Sliced","1A1C2B")
            end    
            scrollElement:AddLabel(5, 3, behavior,0,0,18,"left",true)
            scrollElement:AddButton(210, 0, "", "Reload", 65, 23, "Reload Script", "reload "..behavior, false,"SimpleButton")
            scrollElement:AddButton(275, 0, "Detach|"..behavior, "Detach", 65, 23, "Detach Script", "", false,"SimpleButton")
            scrollWindow:Add(scrollElement)
        end
       
        newWindow:AddScrollWindow(scrollWindow)
        newWindow:AddTextField(30, 375, 260, "Attach", "")
        newWindow:AddButton(305, 375, "Attach", "Attach", 80, 23, "", "", false,"SimpleButton")
 
        newWindow:AddButton(40, 410, "SendMessage", "Send Message", 110, 23, "", "", false,"SimpleButton")
        newWindow:AddButton(155, 410, "FireTimer", "Fire Timer", 110, 23, "", "", false,"SimpleButton")
        newWindow:AddButton(270, 410, "Use", "Use", 110, 23, "", "", false,"SimpleButton")
    else
        local array = GetObjVarsSorted(curInfoObj)
        local scrollWindow = ScrollWindow(25,135,355,250,25)
        for i,entry in pairs(array) do
            local scrollElement = ScrollElement()
            if((i-1) % 2 == 1) then
            scrollElement:AddImage(0,0,"Blank",330,25,"Sliced","1A1C2B")
            end    
 
            local varName = entry.name
            if( varName:len() > 25 ) then
                varName = varName:sub(1,22).."..."
            end
            scrollElement:AddLabel(5, 3, varName,0,0,18)
            local varType = type(entry.value)
            local valueLabel = nil
            if( varType == "userdata" or varType == "table") then
                valueLabel = "["..varType.."]"
            else
                valueLabel = tostring(entry.value)
            end
            scrollElement:AddLabel(200, 3, valueLabel,0,0,18)
 
            local selState = ""
            if(entry.name == selObjVar) then
                selState = "pressed"
            end
               
            scrollElement:AddButton(320, 3, "Select|"..entry.name, "", 0, 18, "", "", false, "Selection",selState)
            scrollWindow:Add(scrollElement)
        end
 
        newWindow:AddScrollWindow(scrollWindow)
        newWindow:AddButton(60, 400, "AddObjVar", "Add", 100, 23, "", "", false,"SimpleButton")
 
        local editState = selObjVar and "" or "disabled"
        newWindow:AddButton(160, 400, "EditObjVar", "Edit", 100, 23, "", "", false,"SimpleButton",editState)
        newWindow:AddButton(260, 400, "DelObjVar", "Delete", 100, 23, "", "", false,"SimpleButton",editState)      
    end    
 
    this:OpenDynamicWindow(newWindow)
end

RegisterEventHandler(EventType.Timer,"DoInfo",
	function (target)
		if(target ~= nil) then
			DoInfo(target)
		end
	end)

-- Allow asynch operation to complete before refreshing the window
function DelayRefresh()
	this:ScheduleTimerDelay(TimeSpan.FromMilliseconds(300),"DoInfo",curInfoObj)
end

RegisterEventHandler(EventType.ClientTargetGameObjResponse, "info",
	function(target,user)
		if not(this:IsDemiGod()) then return end

		if( target == nil ) then
			return
		end

		DoInfo(target)
	end)

function ParseMessageArgs(messageComps)

	table.remove(messageComps,1)

	for i,j in pairs(messageComps) do
		if (tonumber(j) ~= nil) then
			messageComps[i] = tonumber(j)
		elseif (j:match("#")) then
			messageComps[i] = GameObj(tonumber(j:sub(2)))
		end
	end

	return table.unpack(messageComps)
end

RegisterEventHandler(EventType.DynamicWindowResponse,"InfoWindow",
	function (user,returnId,fieldData)
		if(returnId == "AddObjvar") then
			this:SendClientMessage("EnterChat","/setobjvar "..curInfoObj.Id.." ")
		elseif(returnId == "ToggleDebug") then
			if(curInfoObj:HasObjVar("Debug")) then
				curInfoObj:SetObjVar("Debug",true)
			else
				curInfoObj:DelObjVar("Debug")
			end			
		elseif(returnId == "BehaviorsTab") then
			curTab = "Behaviors"
			DoInfo(curInfoObj)
		elseif(returnId == "ObjVarsTab") then
			curTab = "ObjVars"
			DoInfo(curInfoObj)
		elseif(returnId:match("Detach")) then
			if(curInfoObj ~= nil) then
				local behavior = returnId:sub(8)
				curInfoObj:DelModule(behavior)
				DelayRefresh()
			end
		elseif(returnId == "Attach") then
			if(curInfoObj ~= nil and fieldData ~= nil and fieldData.Attach ~= nil and fieldData.Attach ~= "") then
				curInfoObj:AddModule(fieldData.Attach)
				DelayRefresh()
			end
		elseif(returnId:match("Select")) then
			selObjVar = returnId:sub(8)
			DoInfo(curInfoObj)
		elseif(returnId == "EditObjVar") then
			if(selObjVar ~= nil) then
				InitObjVarEditWindow(curInfoObj,selObjVar)
			end
		elseif(returnId == "DelObjVar") then
			if(selObjVar ~= nil) then
				curInfoObj:DelObjVar(selObjVar)
				DelayRefresh()
			end
		elseif(returnId == "AddObjVar") then
			selObjVar = nil
			objVarEditName = nil
			InitObjVarEditWindow(curInfoObj)
		elseif(returnId == "SendMessage" and fieldData ~= nil and fieldData.Attach ~= nil and fieldData.Attach ~= "") then		
			local fieldComps = StringSplit(fieldData.Attach," ")
			if(fieldComps ~= nil and #fieldComps > 0 and curInfoObj:IsValid()) then
				local msgName = fieldComps[1]
				curInfoObj:SendMessage(msgName,ParseMessageArgs(fieldComps))
				this:SystemMessage("Sent message "..msgName.." to "..curInfoObj:GetName())
			end			
		elseif(returnId == "FireTimer" and fieldData ~= nil and fieldData.Attach ~= nil and fieldData.Attach ~= "") then		
			local fieldComps = StringSplit(fieldData.Attach," ")
			if(fieldComps ~= nil and #fieldComps > 0 and curInfoObj:IsValid()) then
				local timerName = fieldComps[1]
				curInfoObj:FireTimer(timerName,ParseMessageArgs(fieldComps))
				this:SystemMessage("Fired timer "..timerName.." on "..curInfoObj:GetName())
			end
		elseif(returnId == "Use") then		
			if(fieldData ~= nil and fieldData.Attach ~= nil and fieldData.Attach ~= "") then
				local fieldComps = StringSplit(fieldData.Attach," ")
				if(fieldComps ~= nil and #fieldComps > 0 and curInfoObj:IsValid()) then
					local useType = fieldComps[1]
					curInfoObj:SendMessage("UseObject",this,useType)
					this:SystemMessage("Fired UseObject type:"..useType.." on "..curInfoObj:GetName())
				end
			else
				curInfoObj:SendMessage("UseObject",this)
				this:SystemMessage("Fired UseObject on "..curInfoObj:GetName())
			end
		elseif(returnId == "Refresh") then
			DoInfo(curInfoObj)
		else
			curInfoObj = nil
			selObjVar = nil
		end
	end)

-- OBJVARS WINDOW

local objVarEditInfo = nil

function InitObjVarEditWindow(targetObj, objVarName)
	local objVarData = nil
	if(objVarName ~= nil and objVarName ~= "") then 
		objVarData = targetObj:GetObjVar(objVarName)
	end
	local objVarType = GetValueType(objVarData)

	objVarEditInfo = { 
		Name = objVarName or "",
		Target = targetObj,
		IsNew = (objVarName == nil),
		Type = objVarType,
		Data = {}
	}
	
	if(objVarType ~= "") then
		objVarEditInfo.Data[objVarType] = objVarData
	end	

	RefreshObjVarEditWindow()
end

function ValidateEditData()
	return objVarEditInfo.Target ~= nil 
		and objVarEditInfo.Target:IsValid() 
		and objVarEditInfo.Name ~= nil 
		and objVarEditInfo.Name ~= ""
		and objVarEditInfo.Type ~= nil
		and objVarEditInfo.Type ~= ""
		and objVarEditInfo.Data[objVarEditInfo.Type] ~= nil
end

function RefreshObjVarEditWindow()
	local windowHeight = 320
	if(objVarEditInfo.Type == "table") then
		windowHeight = 500
	end

	local newWindow = DynamicWindow("ObjVarEditWindow","Edit ObjVar",450,windowHeight)

	newWindow:AddLabel(20, 20, "[F3F781]Name:[-]",600,0,18,"left",false)
	if(objVarEditInfo.IsNew) then
		newWindow:AddTextField(80, 20, 260, "ObjVarName", objVarEditInfo.Name)
	else
		newWindow:AddLabel(80, 20, objVarEditInfo.Name,600,0,18,"left",false)
	end

	newWindow:AddLabel(20, 50, "[F3F781]Type:[-]",600,0,18,"left",false)
	
	local buttonStates = { 
		String = (objVarEditInfo.Type=="string") and "pressed" or "", 
		Number = (objVarEditInfo.Type=="number") and "pressed" or "",
		Bool = (objVarEditInfo.Type=="boolean") and "pressed" or "",
		Table = (objVarEditInfo.Type=="table") and "pressed" or "",
		Loc = (objVarEditInfo.Type=="Loc") and "pressed" or "",
		GameObj = (objVarEditInfo.Type=="GameObj") and "pressed" or "",
		PermanentObj = (objVarEditInfo.Type=="PermanentObj") and "pressed" or ""
	}

	newWindow:AddButton(80, 50, "Type|string", "string", 100, 23, "", "", false,"Selection",buttonStates.String)
	newWindow:AddButton(210, 50, "Type|number", "number", 100, 23, "", "", false,"Selection",buttonStates.Number)
	newWindow:AddButton(340, 50, "Type|Loc", "Loc", 100, 23, "", "", false,"Selection",buttonStates.Loc)
	newWindow:AddButton(80, 80, "Type|boolean", "boolean", 100, 23, "", "", false,"Selection",buttonStates.Bool)
	newWindow:AddButton(210, 80, "Type|table", "table", 100, 23, "", "", false,"Selection",buttonStates.Table)	
	newWindow:AddButton(80, 110, "Type|GameObj", "GameObj", 100, 23, "", "", false,"Selection",buttonStates.GameObj)
	newWindow:AddButton(210, 110, "Type|PermanentObj", "PermanentObj", 100, 23, "", "", false,"Selection",buttonStates.PermanentObj)

	newWindow:AddImage(20,150,"HorizontalWindowDivider",360,1,"Simple")

	if(objVarEditInfo.Type == "string") then
		newWindow:AddLabel(20, 163, "[F3F781]Value:[-]",600,0,18,"left",false)
		newWindow:AddTextField(80, 160, 300, "StringValue", objVarEditInfo.Data.string or "")
	elseif(objVarEditInfo.Type == "number") then
		newWindow:AddLabel(20, 163, "[F3F781]Value:[-]",600,0,18,"left",false)
		newWindow:AddTextField(80, 160, 300, "NumberValue", objVarEditInfo.Data.number and tostring(objVarEditInfo.Data.number) or "")
	elseif(objVarEditInfo.Type == "Loc") then
		newWindow:AddLabel(20, 163, "[F3F781]X:[-]",600,0,18,"left",false)
		newWindow:AddTextField(40, 160, 100, "LocXValue", objVarEditInfo.Data.Loc and tostring(objVarEditInfo.Data.Loc.X) or "")
		newWindow:AddLabel(150, 163, "[F3F781]Y:[-]",600,0,18,"left",false)
		newWindow:AddTextField(170, 160, 100, "LocYValue", objVarEditInfo.Data.Loc and tostring(objVarEditInfo.Data.Loc.Y) or "")
		newWindow:AddLabel(280, 163, "[F3F781]Z:[-]",600,0,18,"left",false)
		newWindow:AddTextField(300, 160, 100, "LocZValue", objVarEditInfo.Data.Loc and tostring(objVarEditInfo.Data.Loc.Z) or "")

		newWindow:AddButton(30, 210, "SelectLoc", "Select Loc", 150, 23, "", "", false,"SimpleButton")
	elseif(objVarEditInfo.Type == "boolean") then		
		newWindow:AddLabel(20, 163, "[F3F781]Value:[-]",600,0,18,"left",false)
		local trueButtonState = (objVarEditInfo.Data.boolean or false) and "pressed" or ""		
		newWindow:AddButton(80, 163, "BoolVal|true", "True", 100, 23, "", "", false,"Selection",trueButtonState)
		local falseButtonState = (objVarEditInfo.Data.boolean or false) and "" or "pressed"
		newWindow:AddButton(210, 163, "BoolVal|false", "False", 100, 23, "", "", false,"Selection",falseButtonState)
	elseif(objVarEditInfo.Type == "GameObj") then
		newWindow:AddLabel(20, 163, "[F3F781]Value:[-]",600,0,18,"left",false)
		newWindow:AddTextField(80, 160, 300, "GameObjValue", objVarEditInfo.Data.GameObj and tostring(objVarEditInfo.Data.GameObj.Id) or "")

		newWindow:AddButton(30, 210, "SelectObj", "Select Object", 150, 23, "", "", false,"SimpleButton")
	elseif(objVarEditInfo.Type == "PermanentObj") then
		newWindow:AddLabel(20, 163, "[F3F781]Value:[-]",600,0,18,"left",false)
		newWindow:AddTextField(80, 160, 300, "PermanentObjValue", objVarEditInfo.Data.PermanentObj and tostring(objVarEditInfo.Data.PermanentObj.Id) or "")

		newWindow:AddButton(30, 210, "SelectPermObj", "Select Object", 150, 23, "", "", false,"SimpleButton")
	elseif(objVarEditInfo.Type == "table") then
		if(objVarEditInfo.Data.table == nil) then
			newWindow:AddLabel(20, 163, "Table editing coming soon (tm)",0,0,18,"left",false)
		else
			newWindow:AddLabel(20, 163, DumpTable(objVarEditInfo.Data.table),400,230,18,"",true)
		end
	end

	if(objVarEditInfo.Type ~= "table") then
		newWindow:AddButton(310, windowHeight - 90, "SaveObjVar", "Save", 100, 23, "", "", true,"SimpleButton")
	end

	this:OpenDynamicWindow(newWindow)
end

RegisterEventHandler(EventType.DynamicWindowResponse,"ObjVarEditWindow",
	function (user,returnId,fieldData)
		if(fieldData ~= nil) then
			if(fieldData.ObjVarName ~= nil) then
				objVarEditInfo.Name = fieldData.ObjVarName
			end

			if(fieldData.StringValue ~= nil) then
				objVarEditInfo.Data.string = fieldData.StringValue
			end

			if(fieldData.NumberValue ~= nil) then
				objVarEditInfo.Data.number = tonumber(fieldData.NumberValue)
			end

			if(fieldData.LocXValue ~= nil and fieldData.LocYValue ~= nil and fieldData.LocZValue ~= nil) then
				local locX, locY, locZ = tonumber(fieldData.LocXValue), tonumber(fieldData.LocYValue), tonumber(fieldData.LocZValue)
				if(locX ~= nil and locY ~= nil and locZ ~= nil) then
					objVarEditInfo.Data.Loc = Loc(locX, locY, locZ)													
				end
			end

			if(fieldData.GameObjValue ~= nil) then
				local id = tonumber(fieldData.GameObjValue)
				if(id ~= nil) then
					objVarEditInfo.Data.GameObj = GameObj(id)
				end
			end

			if(fieldData.PermanentObjValue ~= nil) then
				local id = tonumber(fieldData.PermanentObjValue)
				if(id ~= nil) then
					objVarEditInfo.Data.PermanentObj = GameObj(id)
				end
			end
		end

		if(returnId:match("Type")) then
			objVarEditInfo.Type = returnId:sub(6)
			RefreshObjVarEditWindow()
		elseif(returnId == "SaveObjVar") then			
			if(ValidateEditData()) then
				local objVarData = objVarEditInfo.Data[objVarEditInfo.Type]
				objVarEditInfo.Target:SetObjVar(objVarEditInfo.Name,objVarData)
				this:SystemMessage("Objvar "..objVarEditInfo.Name.." set on target "..objVarEditInfo.Target:GetName().." ("..objVarEditInfo.Target.Id..")")
				-- update info window if its open
				if(curInfoObj ~= nil) then
					DelayRefresh()
				end
			else
				this:SystemMessage("Objvar edit data failed validation. Try again.")
			end
		elseif(returnId == "SelectLoc") then
			this:RequestClientTargetLoc(this, "SelectObjVarLoc")
		elseif(returnId == "SelectObj") then
			this:RequestClientTargetGameObj(this, "SelectObjVarObj")
		elseif(returnId == "SelectPermObj") then
			this:RequestClientTargetAnyObj(this, "SelectObjVarPermObj")
		elseif(returnId:match("BoolVal")) then			
			if(objVarEditInfo ~= nil) then
				objVarEditInfo.Data.boolean = (returnId:match("true") ~= nil)
				RefreshObjVarEditWindow()
			end
		end
	end)

RegisterEventHandler(EventType.ClientTargetLocResponse, "SelectObjVarLoc", 
	function (success, targetLoc)	
		if(success) then
			if(objVarEditInfo ~= nil) then
				objVarEditInfo.Data.Loc = targetLoc
				RefreshObjVarEditWindow()
			end
		end
	end)

RegisterEventHandler(EventType.ClientTargetGameObjResponse, "SelectObjVarObj", 
    function(target,user)
    	if(target ~= nil and objVarEditInfo ~= nil) then
    		objVarEditInfo.Data.GameObj = target
    		RefreshObjVarEditWindow()
    	end
    end)

RegisterEventHandler(EventType.ClientTargetAnyObjResponse, "SelectObjVarPermObj", 
    function(target,user)
    	if(target ~= nil and objVarEditInfo ~= nil) then
    		if not(target:IsPermanent()) then
    			this:SystemMessage("You must select a permanent object.")
    		else
	    		objVarEditInfo.Data.PermanentObj = target
    			RefreshObjVarEditWindow()
    		end
    	end
    end)
	

-------------------------------------------------------------------------
-- MOD CODE
-------------------------------------------------------------------------
	



-- Mr Staples

--require 'staples.attachScriptOverride' -- overrides the attach command from the info window needed for custom scenes

OverrideEventHandler("scriptcommands_UI_info", EventType.DynamicWindowResponse,"InfoWindow",
	function (user,returnId,fieldData)
		if(returnId == "AddObjvar") then
			this:SendClientMessage("EnterChat","/setobjvar "..curInfoObj.Id.." ")
		elseif(returnId == "ToggleDebug") then
			if(curInfoObj:HasObjVar("Debug")) then
				curInfoObj:SetObjVar("Debug",true)
			else
				curInfoObj:DelObjVar("Debug")
			end			
		elseif(returnId == "BehaviorsTab") then
			curTab = "Behaviors"
			DoInfo(curInfoObj)
		elseif(returnId == "ObjVarsTab") then
			curTab = "ObjVars"
			DoInfo(curInfoObj)
		elseif(returnId:match("Detach")) then
			if(curInfoObj ~= nil) then
				local behavior = returnId:sub(8)
				curInfoObj:DelModule(behavior)
				DelayRefresh()
			end
		elseif(returnId == "Attach") then
			if(curInfoObj ~= nil and fieldData ~= nil and fieldData.Attach ~= nil and fieldData.Attach ~= "") then
				curInfoObj:AddModule(fieldData.Attach)

--------------------Mod is here-----------------------				
				if curInfoObj:HasObjVar("sceneName") then -- and curInfoObj:HasModule(fieldData.Attach) then
					local attachedScripts = " "
			
					if curInfoObj:HasObjVar("customScripts") then
						attachedScripts = curInfoObj:GetObjVar("customScripts") .. "," .. fieldData.Attach 
					else
						attachedScripts = fieldData.Attach 
					end

					curInfoObj:SetObjVar("customScripts", attachedScripts)
				end
--------------------Mod is here-----------------------				

				DelayRefresh()
			end
		elseif(returnId:match("Select")) then
			selObjVar = returnId:sub(8)
			DoInfo(curInfoObj)
		elseif(returnId == "EditObjVar") then
			if(selObjVar ~= nil) then
				InitObjVarEditWindow(curInfoObj,selObjVar)
			end
		elseif(returnId == "DelObjVar") then
			if(selObjVar ~= nil) then
				curInfoObj:DelObjVar(selObjVar)
				DelayRefresh()
			end
		elseif(returnId == "AddObjVar") then
			selObjVar = nil
			objVarEditName = nil
			InitObjVarEditWindow(curInfoObj)
		elseif(returnId == "SendMessage" and fieldData ~= nil and fieldData.Attach ~= nil and fieldData.Attach ~= "") then		
			local fieldComps = StringSplit(fieldData.Attach," ")
			if(fieldComps ~= nil and #fieldComps > 0 and curInfoObj:IsValid()) then
				local msgName = fieldComps[1]
				curInfoObj:SendMessage(msgName,ParseMessageArgs(fieldComps))
				this:SystemMessage("Sent message "..msgName.." to "..curInfoObj:GetName())
			end			
		elseif(returnId == "FireTimer" and fieldData ~= nil and fieldData.Attach ~= nil and fieldData.Attach ~= "") then		
			local fieldComps = StringSplit(fieldData.Attach," ")
			if(fieldComps ~= nil and #fieldComps > 0 and curInfoObj:IsValid()) then
				local timerName = fieldComps[1]
				curInfoObj:FireTimer(timerName,ParseMessageArgs(fieldComps))
				this:SystemMessage("Fired timer "..timerName.." on "..curInfoObj:GetName())
			end
		elseif(returnId == "Use") then		
			if(fieldData ~= nil and fieldData.Attach ~= nil and fieldData.Attach ~= "") then
				local fieldComps = StringSplit(fieldData.Attach," ")
				if(fieldComps ~= nil and #fieldComps > 0 and curInfoObj:IsValid()) then
					local useType = fieldComps[1]
					curInfoObj:SendMessage("UseObject",this,useType)
					this:SystemMessage("Fired UseObject type:"..useType.." on "..curInfoObj:GetName())
				end
			else
				curInfoObj:SendMessage("UseObject",this)
				this:SystemMessage("Fired UseObject on "..curInfoObj:GetName())
			end
		elseif(returnId == "Refresh") then
			DoInfo(curInfoObj)
		else
			curInfoObj = nil
			selObjVar = nil
		end
	end)