-- dye
-- Definition: A dye is an object which allows to color another object (=set the hue)
-- hue definitions are included from incl_colors:
require 'incl_colors'
require 'incl_dyetypes'



-- events: 
	-- Doubleclick == use
-- Message Events:
	-- 
	
-- Data Members: 
	-- color: The hex code string for the color
	-- uses : Uses left
-- Functions: 
	-- DyeOn(target) 		: Applies a SetHue, mixing the color to the Preexisting Hue of the target (using a mean value)
	-- PourOn(target) 	: Applies a SetHue, overriding the original Hue of the target
	-- MixInto (Apply to a Mix Bottle Object )	
	-- Spill(target)  	: The mess if something fails (Failing a Dex/Agi Stat Check or later a Skill Check)
	-- init(color, uses): 

------------------------------------------------
--DEFAULTS
------------------------------------------------

-- set your ObjVars !!!!!

---------------- TESTING
	--- First Setting Defaults, Then overriding
	--this:SetObjVar("DyeColor"	  , "FF0000"	)	-- default red Dye
	--this:SetObjVar("Uses"		  , 5 			)	-- five uses before the dye is used up
	--this:SetObjVar("SatCost"	  , 3 			)	-- uses cost for total color override (Saturate)
	--this:SetObjVar("DyeType"	  , "cloth" 	)  	-- dyetype refers to a list of allowed object templates 
													-- to dye or it is "omni" for all. See 'incl_dyetypes.lua'.
	--this:SetObjVar("NamePreFix" , "Dye ("		)   -- Name Prefix   color name goes between pre- and postfix
	--this:SetObjVar("NamePostFix", 	    ")"	)   -- Name Postfix

		


------------------------------------------------
-- COLOR APPLICATION
------------------------------------------------
	-- Dye On mixes with preexisting color
	function HandleDyeOn(target)
		DebugMessage("HandleDyeOn Target: "..tostring(target:GetName())..":"..tostring(target.Id))
		-- Dye-Target compatibility check
		if not (PermCheck(target)) then return end

		local result = MixHues(target:GetHue(), this:GetObjVar("DyeColor"), 1, 1)
		--DebugMessage("DyeOn: "..tostring(target:GetName())..":"..tostring(target.Id).." -> "..target:GetHue().."=>"..curHue )

		target:SetHue( result )
		-- Used up test
		local u = this:GetObjVar("Uses")
		if (u<=1) then this:Destroy() end
		this:SetObjVar("Uses", u-1)

		this:SetSharedObjectProperty("TooltipString", "Remaining Uses: "..this:GetObjVar("Uses") )

		DebugMessage("HandleDyeOn 'this' is "..tostring(this:GetName()..":"..tostring(this.Id)))
	end

	-- SaturateOn overrides preexisting color at a cost (sat_cost)
	function HandleSaturateOn(target)
		DebugMessage("HandleSaturateOn Target: "..tostring(target:GetName())..":"..tostring(target.Id))
		-- Dye-Target compatibility check
		if not (PermCheck(target)) then return end

		if this:GetObjVar("Uses")>=this:GetObjVar("SatCost") then -- make sure we have enough dye to Saturate
			target:SetHue(this:GetObjVar("DyeColor") )
			
			-- Used up test
			local u = this:GetObjVar("Uses")
			if (u<=this:GetObjVar("SatCost")) then this:Destroy() end
			this:SetObjVar("Uses",this:GetObjVar("Uses") - this:GetObjVar("SatCost"))

			this:SetSharedObjectProperty("TooltipString", "Remaining Uses: "..this:GetObjVar("Uses") )

		end

	end

	-- TODO:  A color "accident" ...
	function HandleSpill(user)
		-- body
	end

------------------------------------------------
-- Getters and Setters
------------------------------------------------
	-- We use ObjVars !!
	function SetDyeColor( newDyeColor )
		DebugMessage("SetDyeColor: "..this:GetName()..":"..this.Id)
		this:SetObjVar("DyeColor", newDyeColor)
	end

	-- adjusts object name after a color mix
	function HandleAdjust(newDye)
		DebugMessage("HandleAdjust: "..this:GetName()..":"..this.Id.." newDye:"..string.format("%06X",tonumber(newDye,16)) )
		-- try to get a real color name
		local name = GetColorFromDye(newDye)
		if (name == nil) then name = newDye end
		this:SetName( this:GetObjVar("NamePreFix") 	..name	..this:GetObjVar("NamePostFix")		)
		this:SetHue(newDye)
		this:SetSharedObjectProperty("TooltipString", this:GetName().."\n".."Remaining Uses: "..this:GetObjVar("Uses") )

	end

------------------------------------------------
-- Utility Functions
------------------------------------------------
	---"COLORS" = Azure, Gold, Red, etc
	---- "DYES" = FFB819 etc

	-- Returns the Dye value frum the COLORS DICT
	function GetColorFromDye( s )
		return DYES["x"..string.upper(s)]
	end

	-- Returns the Color name from the DYES DICT 
	function GetDyeFromColor( s )
		return COLORS[s]
	end

	-- Dye - target compatibility check
	function PermCheck(target)
		local template = target:GetCreationTemplateId()
		--DebugMessage("PermCheck: "..IdStr(target).." TemplateId:"..template)
		if this:GetObjVar("DyeType") =="omni" then 
			DebugMessage("Dye:PermCheck 'omni' Dyetype: "..this:GetObjVar("DyeType")..", template:"..template..", (DYETYPES[DyeType])[template]"..tostring((DYETYPES[this:GetObjVar("DyeType")])[template])..", target:"..IdStr(target))
			return true 
		end -- "omni" can dye everything

		if ( (DYETYPES[this:GetObjVar("DyeType")])[template] ~= true) then -- if not in list or not true= no action
			DebugMessage("Dye:PermCheck 'wrong' Dyetype: "..this:GetObjVar("DyeType")..", template:"..template..", (DYETYPES[DyeType])[template]"..tostring((DYETYPES[this:GetObjVar("DyeType")])[template])..", target:"..IdStr(target))
			return false 
		end
            DebugMessage("Dye:PermCheck 'good' Dyetype: "..this:GetObjVar("DyeType")..", template:"..template..", (DYETYPES[DyeType])[template]"..tostring((DYETYPES[this:GetObjVar("DyeType")])[template])..", target:"..IdStr(target))
		return true -- template is in list and thus can be dyed
	end

	--Debug Helper
	function IdStr(target)
		return tostring(target:GetName())..":"..tostring(target.Id)
	end
------------------------------------------------
-- Mixing
------------------------------------------------
	-- Color Mixing
	-- the remaining uses of the flasks determine the weight of each color and it gets split up between them
	-- uses remain the same

	function HandleMix(DyeFlask)
		DebugMessage("HandleMix DyeFlask="..tostring(DyeFlask:GetName()..":"..tostring(DyeFlask.Id)))

		if this:GetObjVar("DyeType") ~= DyeFlask:GetObjVar("DyeType") then return end

		local newDye = MixHues(     this:GetObjVar("DyeColor"), 
									DyeFlask:GetObjVar("DyeColor"),
									    this:GetObjVar("Uses"),
									DyeFlask:GetObjVar("Uses")
									)
		-- Adjusting to new Dye
		    this:SetObjVar( "DyeColor", newDye)
		DyeFlask:SetObjVar( "DyeColor", newDye)
		--- Adjusting Name and Hue
			this:SendMessage("Adjust", newDye)
		DyeFlask:SendMessage("Adjust", newDye)

	end


	-- weighed color mix
	function MixHues(hue1,hue2,weight1, weight2)
		if weight1==nil then weight1=1 end
		if weight2==nil then weight2=1 end

        local hue1red = tonumber(string.sub(hue1,1,2),16)*weight1
        local hue1green = tonumber(string.sub(hue1,3,4),16)*weight1
        local hue1blue = tonumber(string.sub(hue1,5,6),16)*weight1
        
        local hue2red = tonumber(string.sub(hue2,1,2),16)*weight2
        local hue2green = tonumber(string.sub(hue2,3,4),16)*weight2
        local hue2blue = tonumber(string.sub(hue2,5,6),16)*weight2
       	local divisor = weight1 + weight2
        local newred =  math.floor( (hue1red   + hue2red) 	/ divisor)
        local newgreen =math.floor( (hue1green + hue2green) / divisor)
        local newblue = math.floor( (hue1blue  + hue2blue) 	/ divisor)       
       
        return string.format("%02X%02X%02X",newred,newgreen,newblue)
	end
------------------------------------------------
-- Interaction
------------------------------------------------

	function HandleUseDye (user)
		user:SystemMessage("You start working with the dye.")
		local menuItems = {
		"Dye target",
		"Saturate target",
		"Mix colors",
		--"Adjust Name",
		}
		user:RequestContextMenu(this,"dye_contextmenu", menuItems)	
	end

	function HandleMenuChoice( user, menuIndex )
    if( menuIndex == 0 ) then return end
	    
	  if( user == nil or not(user:IsValid())) then return end
	    
	  --if( user:DistanceFrom(this)==nil or user:DistanceFrom(this) > OBJECT_INTERACTION_RANGE) then return end

		if( menuIndex == 1 ) then
		  	--Initiate Dye Target
		  	user:SystemMessage("Dye Target")
		  	user:RequestClientTargetGameObj(this,"DyeOn")
		  elseif( menuIndex == 2 ) then	
		  	-- Initiate Saturate Target
		  	user:SystemMessage("Saturate Target")
		  	user:RequestClientTargetGameObj(this,"SaturateOn")
		  elseif( menuIndex == 3 ) then
		  	-- Initiate Mix Colors
		  	user:SystemMessage("Mix Colors")
		  	user:RequestClientTargetGameObj(this,"Mix")
		   	--user:SystemMessage ("Un-Subscribing All" .. tostring(cronObj:SendMessage("CronRemoveObject", this.Id) ))
		  --elseif( menuIndex == 4 ) then
		  --	-- Initiate Mix Colors
		  --	user:SystemMessage("Re-Initialize Flask")
		  --	this:SendMessage("Adjust", this:GetObjVar("DyeColor"))
		end

	end
------------------------------------------------
-- EventHandlers
------------------------------------------------
	RegisterEventHandler(EventType.Message, "UseObject", HandleUseDye)
	RegisterEventHandler(EventType.ContextMenuResponse,"dye_contextmenu", HandleMenuChoice)

	RegisterEventHandler(EventType.ClientTargetGameObjResponse, "DyeOn"		 , HandleDyeOn		)
	RegisterEventHandler(EventType.ClientTargetGameObjResponse, "SaturateOn" , HandleSaturateOn	)
	RegisterEventHandler(EventType.ClientTargetGameObjResponse, "Mix"		 , HandleMix		)

	RegisterEventHandler(EventType.Message, "Adjust", HandleAdjust)
	RegisterEventHandler(EventType.Message, "SetDyeColor", SetDyeColor)

	RegisterEventHandler(EventType.Message, "Spill"			 , HandleSpill	)

------------------------------------------------
-- Initialize
------------------------------------------------
	
	--HandleAdjust(this:GetObjVar("DyeColor"))


	